@@ -888,7 +888,7 @@ private:
 
 	bool readActiveWorkspace(void);
 	bool readActiveWindow(void);
-	bool readClientListStacking(void);
+	bool readClientList(void);
 	bool readDesktopNames(void);
 	void readRootProperties(void);
 	bool readRootProperty(Atom atom);
@@ -928,7 +928,7 @@ WmState::read(void)
 {
 	readActiveWorkspace();
 	readActiveWindow();
-	readClientListStacking();
+	readClientList();
 	readDesktopNames();
 	readRootProperties();
 }
@@ -951,7 +951,7 @@ WmState::handlePropertyNotify(XPropertyEvent *ev)
 		} else if (ev->atom == X11::getAtom(NET_ACTIVE_WINDOW)) {
 			updated = readActiveWindow();
 		} else if (ev->atom == X11::getAtom(NET_CLIENT_LIST)) {
-			updated = readClientListStacking();
+			updated = readClientList();
 		} else if (ev->atom == X11::getAtom(XROOTPMAP_ID)) {
 			observation = &_xrootpmap_id_changed;
 		} else if (ev->atom == X11::getAtom(PEKWM_THEME)) {
@@ -1027,27 +1027,29 @@ WmState::readActiveWindow(void)
 }
 
 bool
-WmState::readClientListStacking(void)
+WmState::readClientList(void)
 {
 	ulong actual;
 	Window *windows;
-	if (! X11::getProperty(X11::getRoot(),
-			       X11::getAtom(NET_CLIENT_LIST),
-			       XA_WINDOW, 0,
-			       reinterpret_cast<uchar**>(&windows), &actual)) {
-		P_TRACE("failed to read _NET_CLIENT_LIST");
-		return false;
-	}
 
 	client_info_vector old_clients = _clients;
 	_clients.clear();
-	for (uint i = 0; i < actual; i++) {
-		ClientInfo *client_info = popClientInfo(windows[i], old_clients);
-		if (client_info == nullptr) {
-			_clients.push_back(new ClientInfo(windows[i]));
-		} else {
-			_clients.push_back(client_info);
+
+	if (X11::getProperty(X11::getRoot(),
+			     X11::getAtom(NET_CLIENT_LIST),
+			     XA_WINDOW, 0,
+			     reinterpret_cast<uchar**>(&windows), &actual)) {
+		P_TRACE("read _NET_CLIENT_LIST, " << actual << " windows");
+		for (uint i = 0; i < actual; i++) {
+			ClientInfo *client_info =
+			  popClientInfo(windows[i], old_clients);
+			if (client_info == nullptr) {
+				_clients.push_back(new ClientInfo(windows[i]));
+			} else {
+				_clients.push_back(client_info);
+			}
 		}
+		X11::free(windows);
 	}
 
 	client_info_it it = old_clients.begin();
@@ -1055,10 +1057,7 @@ WmState::readClientListStacking(void)
 		delete *it;
 	}
 
-	X11::free(windows);
-
-	P_TRACE("read _NET_CLIENT_LIST, " << actual << " windows");
-	return true;
+	return old_clients.size() > 0;
 }
 
 bool
@@ -2117,6 +2116,9 @@ public:
 	virtual void render(Render& rend);
 
 private:
+	void renderFixed(Render& rend);
+	void renderScaled(Render& rend);
+
 	void load(void);
 	bool loadImage(const std::string& icon_name);
 	void parseIcon(const CfgParser::Entry* section);
@@ -2130,6 +2132,8 @@ private:
 	std::string _ext;
 	/** Regex transform of observed field */
 	RegexString _transform;
+	/** If true, scale icon square to fit panel height */
+	bool _scale;
 
 	/** current loaded icon, matching _icon_name. */
 	PImage* _icon;
@@ -2144,6 +2148,7 @@ IconWidget::IconWidget(const PanelTheme& theme,
 	: PanelWidget(theme, size_req),
 	  _var_data(var_data),
 	  _field(field),
+	  _scale(false),
 	  _icon(nullptr)
 {
 	parseIcon(section);
@@ -2173,7 +2178,10 @@ IconWidget::notify(Observable *, Observation *observation)
 uint
 IconWidget::getRequiredSize(void) const
 {
-	return _theme.getHeight();
+	if (_scale || _icon == nullptr) {
+		return _theme.getHeight();
+	}
+	return _icon->getWidth() + 2;
 }
 
 void
@@ -2181,11 +2189,35 @@ IconWidget::render(Render& rend)
 {
 	PanelWidget::render(rend);
 	if (_icon == nullptr) {
-		return;
+		// do nothing, no icon to render
+	} else if (_scale) {
+		renderScaled(rend);
+	} else {
+		renderFixed(rend);
 	}
+}
 
-	uint height = _theme.getHeight() - 2;
-	_icon->draw(rend, getX(), 1, height, height);
+void
+IconWidget::renderFixed(Render& rend)
+{
+	uint height, width;
+	uint height_avail = _theme.getHeight() - 2;
+	if (_icon->getHeight() > height_avail) {
+		float aspect = (float) height_avail / _icon->getHeight();
+		height = height_avail;
+		width = _icon->getWidth() * aspect;
+	} else {
+		height = _icon->getHeight();
+		width = _icon->getWidth();
+	}
+	_icon->draw(rend, getX() + 1, 1, width, height);
+}
+
+void
+IconWidget::renderScaled(Render& rend)
+{
+	uint side = _theme.getHeight() - 2;
+	_icon->draw(rend, getX() + 1, 1, side, side);
 }
 
 void
@@ -2231,6 +2263,7 @@ IconWidget::parseIcon(const CfgParser::Entry* section)
 	std::vector<CfgParserKey*> keys;
 	keys.push_back(new CfgParserKeyString("ICON", name));
 	keys.push_back(new CfgParserKeyString("TRANSFORM", transform));
+	keys.push_back(new CfgParserKeyBool("SCALE", _scale));
 	section->parseKeyValues(keys.begin(), keys.end());
 	std::for_each(keys.begin(), keys.end(), Util::Free<CfgParserKey*>());
 
@@ -2244,6 +2277,9 @@ IconWidget::parseIcon(const CfgParser::Entry* section)
 	if (transform.size() > 0) {
 		_transform.parse_ed_s(transform);
 	}
+
+	// inital load of image to get size request right
+	load();
 }
 
 /**
